/**
 * Copyright 1998-2007, CHISEL Group, University of Victoria, Victoria, BC, Canada.
 * All rights reserved.
 */
package ca.uvic.csr.shrimp.adapter;

import java.util.Properties;

import ca.uvic.csr.shrimp.DataBean.DataBean;
import ca.uvic.csr.shrimp.DisplayBean.DisplayBean;
import ca.uvic.csr.shrimp.FilterBean.FilterBean;
import ca.uvic.csr.shrimp.SelectorBean.SelectorBean;
import ca.uvic.csr.shrimp.ShrimpApplication.ApplicationAccessor;
import ca.uvic.csr.shrimp.ShrimpApplication.BeanNotFoundException;
import ca.uvic.csr.shrimp.ShrimpApplication.ShrimpApplication;
import ca.uvic.csr.shrimp.ShrimpApplication.ShrimpToolNotFoundException;
import ca.uvic.csr.shrimp.ShrimpProject.ShrimpProject;
import ca.uvic.csr.shrimp.ShrimpTool.ShrimpTool;
import ca.uvic.csr.shrimp.gui.ActionManager.CheckBoxAction;
import ca.uvic.csr.shrimp.gui.ShrimpView.ShrimpView;
import ca.uvic.csr.shrimp.gui.options.DataFiltersOptionsPanel;
import ca.uvic.csr.shrimp.gui.options.GeneralOptionsPanel;
import ca.uvic.csr.shrimp.gui.options.LabelOptionsPanel;
import ca.uvic.csr.shrimp.gui.options.LayoutOptionsPanel;
import ca.uvic.csr.shrimp.gui.options.RenderingOptionsPanel;
import ca.uvic.csr.shrimp.gui.options.ShrimpOptionsDialog;
import ca.uvic.csr.shrimp.gui.options.UserControlOptions;
import ca.uvic.csr.shrimp.resource.ResourceHandler;
import ca.uvic.csr.shrimp.usercontrols.DefaultProjectAction;
import ca.uvic.csr.shrimp.usercontrols.ToggleLongToolTipAdapter;

/**
 * Initializes the Shrimp Options Dialog.  This includes initializing all the Option Panels
 * as well as the General Options panel
 * 
 * @author Jeff Michaud
 */
public class OpenShrimpOptionsAdapter extends DefaultProjectAction {

	private ShrimpOptionsDialog shrimpOptionsDialog;
	private GeneralOptionsPanel generalOptionsPanel;
	private int selectedTabIndex = 0;

	public OpenShrimpOptionsAdapter(String actionName, ShrimpProject project) {
		super(actionName, ResourceHandler.getIcon("icon_options.gif"), project);
	}
	
	public void startAction() {
		try {
			ShrimpProject project = getProject();
			ShrimpView shrimpView = (ShrimpView) project.getTool(ShrimpProject.SHRIMP_VIEW);
			DisplayBean displayBean = (DisplayBean) shrimpView.getBean(ShrimpTool.DISPLAY_BEAN);
			DataBean dataBean = (DataBean) project.getBean(ShrimpProject.DATA_BEAN);
            FilterBean dataFilterBean = (FilterBean) project.getBean(ShrimpProject.DATA_FILTER_BEAN);
			SelectorBean selectorBean = (SelectorBean) shrimpView.getBean(ShrimpTool.SELECTOR_BEAN);

			ShrimpApplication app = ApplicationAccessor.getApplication();
			configureGeneralOptions(app, shrimpView, displayBean);
			
			shrimpOptionsDialog = new ShrimpOptionsDialog(app.getParentFrame(), true);
			shrimpOptionsDialog.addTab("General", generalOptionsPanel);
			shrimpOptionsDialog.addTab("Labels", new LabelOptionsPanel(displayBean, selectorBean));
			shrimpOptionsDialog.addTab("Layouts", new LayoutOptionsPanel(displayBean));
			shrimpOptionsDialog.addTab("User Controls", new UserControlOptions(app));		
			shrimpOptionsDialog.addTab("Data Filters", new DataFiltersOptionsPanel(project, dataBean, dataFilterBean));
			shrimpOptionsDialog.addTab("Rendering", new RenderingOptionsPanel(ShrimpProject.SHRIMP_VIEW, project));
			
			// load the selected tab
			shrimpOptionsDialog.setSelectedTabIndex(selectedTabIndex);
			
			// show the dialog (blocks here)
			shrimpOptionsDialog.setVisible(true);
			
			// save the selected tab
			selectedTabIndex = shrimpOptionsDialog.getSelectedTabIndex();
		} catch(ShrimpToolNotFoundException stnfe) {
			stnfe.printStackTrace();
		} catch (BeanNotFoundException e) {
			e.printStackTrace();
		}
	}

	//adds checkbox actions to the general options tab
	private void configureGeneralOptions(ShrimpApplication app, ShrimpView shrimpView, final DisplayBean displayBean) {
		//configure the options dialog
		ShrimpProject project = getProject();
		generalOptionsPanel = new GeneralOptionsPanel(project);
		
		final Properties props = app.getProperties(); 
		
		//Long Tool Tips
		ToggleLongToolTipAdapter longToolTipAdapter = new ToggleLongToolTipAdapter(project, shrimpView);		
		shrimpView.addUserControl(longToolTipAdapter);
		
		//Arrow Heads
		CheckBoxAction arrowHeadAction = new CheckBoxAction("Use Arrow Heads", project) {
			public void startAction () {							
				displayBean.setUsingArrowHeads(getChecked());
				displayBean.repaint();
			}
		};	
		arrowHeadAction.setChecked(displayBean.getUsingArrowHeads());	
		
		//Return to previous view dialog
		CheckBoxAction returnToPreviousView = new CheckBoxAction("Display 'Return to Previous View' dialog on startup") {
			public void startAction() {
				props.setProperty(ShrimpApplication.SHOW_RETURN_TO_PREVIOUS_DIALOG_KEY, "" + getChecked());				
			}
		};			
		String showLoadPreviousDialogStr = props.getProperty(ShrimpApplication.SHOW_RETURN_TO_PREVIOUS_DIALOG_KEY, "true");
		returnToPreviousView.setChecked("true".equalsIgnoreCase(showLoadPreviousDialogStr));
				
		//Panel mode change warning dialog		
		CheckBoxAction showAllDescendentsWarning = new CheckBoxAction("Display 'Show Panel Mode Change' warning") {
			public void startAction() {
				props.setProperty(PanelModeChangeAdapter.WARNING_KEY, "" + getChecked());								
			}
		};			
		boolean showWarning = true;
		String showWarningStr = props.getProperty(PanelModeChangeAdapter.WARNING_KEY, "True");	
		showWarning = Boolean.valueOf(showWarningStr).booleanValue();
		showAllDescendentsWarning.setChecked(showWarning);
		
		CheckBoxAction showScrollbars = new CheckBoxAction("Show scrollbars in the main Shrimp view (requires re-opening current project)") {
			public void startAction() {
				props.setProperty(ShrimpApplication.USE_SCROLLPANE, "" + getChecked());
			}
		};
		String shownStr = props.getProperty(ShrimpApplication.USE_SCROLLPANE, "true");
		showScrollbars.setChecked("true".equalsIgnoreCase(shownStr));
		
		generalOptionsPanel.addCheckBoxAction(longToolTipAdapter);		
		generalOptionsPanel.addCheckBoxAction(arrowHeadAction);
		generalOptionsPanel.addCheckBoxAction(returnToPreviousView);
		generalOptionsPanel.addCheckBoxAction(showAllDescendentsWarning);
		generalOptionsPanel.addCheckBoxAction(showScrollbars);		
	}

}
