/**
 * Copyright 1998-2007, CHISEL Group, University of Victoria, Victoria, BC, Canada.
 * All rights reserved.
 */
package ca.uvic.csr.shrimp.jambalaya;

import java.awt.dnd.DnDConstants;
import java.awt.dnd.DragSource;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import javax.swing.Action;
import javax.swing.JPopupMenu;
import javax.swing.JTree;
import javax.swing.tree.TreeCellRenderer;

import ca.uvic.csr.shrimp.ShrimpApplication.BeanNotFoundException;
import ca.uvic.csr.shrimp.ShrimpProject.ShrimpProject;
import ca.uvic.csr.shrimp.jambalaya.ProtegeDataBean.ProtegeDataBean;
import edu.stanford.smi.protege.model.Cls;
import edu.stanford.smi.protege.model.Frame;
import edu.stanford.smi.protege.model.Project;
import edu.stanford.smi.protege.ui.ClsesPanel;
import edu.stanford.smi.protege.ui.ClsesTreeDragSourceListener;
import edu.stanford.smi.protege.ui.FrameRenderer;
import edu.stanford.smi.protege.ui.SubclassPane;


/**
 * Extension of the ClsesPanel to use a JambalayaSubclasspane instead of a Subclasspane. 
 * 
 * @author Rob Lintern, Chris Callendar
 */
public class JambalayaClsesPanel extends ClsesPanel {

	private ArrayList clsTreeActions = null;
	
	// TODO selector bean to provide the selected ShrimpNodes
	// sometimes the selection synchronization from the clsTree and Jambalaya view doesn't
	// work because if the selected class isn't visible in Jambalaya, it won't be selected
	
	//private SelectorBean selector;
	
    public JambalayaClsesPanel(Project project) {
        super(project);
    
        clsTreeActions = new ArrayList(5);
        clsTreeActions.add(new NeighbourhoodAction());
        clsTreeActions.add(new SubclassTreeAction());
        clsTreeActions.add(new SuperclassTreeAction());
    }

    protected SubclassPane createSubclassPane(Action viewAction, Cls root, Action createAction, Action action) {
        return new JambalayaSubclassPane(root, createAction, action);
    }
    
    public void addClsTreeContextMenuAction(Action action) {
    	if (!clsTreeActions.contains(action)) {
            clsTreeActions.add(action);
    	}
    }
    
    public boolean removeClsTreeContextMenuAction(Action action) {
    	return clsTreeActions.remove(action);
    }
    
    // JambalayaSubclassPane Inner Class
    
    public class JambalayaSubclassPane extends SubclassPane {
        
    	public JambalayaSubclassPane(Cls rootCls, Action createCls, Action deleteCls) {
            super(null, rootCls, createCls, deleteCls);
            
            // TODO make this a preference? Preferences are not very well organized...
            // @tag Jambalaya(ShowInstanceCount)
            JTree tree = (JTree)getSelectable();
            TreeCellRenderer renderer = tree.getCellRenderer();
            if (renderer instanceof FrameRenderer) {
				((FrameRenderer) renderer).setDisplayDirectInstanceCount(true);				
			}
    	}
        
        /** 
         * Overridden so that we can drag and drop :THING and also so that
         * we can't actually drag and drop within the clses tree itself 
         * changing the knowledgebase accidentally
         */
        protected void setupDragAndDrop() {
            DragSource.getDefaultDragSource().createDefaultDragGestureRecognizer((JTree) getSelectable(), 
            		DnDConstants.ACTION_COPY_OR_MOVE, new JambalayaClsesTreeDragSourceListener());
        }
        
        /**
         * Overridden to add the Neighbourhood, Subclass, and Superclass actions.
         */
        protected JPopupMenu createPopupMenu() {
        	JPopupMenu menu = super.createPopupMenu();
        	
        	// add the Jambalaya resource actions
        	menu.addSeparator();
        	for (Iterator iter = clsTreeActions.iterator(); iter.hasNext(); ) {
        		menu.add((Action)iter.next());	
        	}
        	
        	return menu;
        }
    }


    // JambalayaClsesTreeDragSourceListener Inner Class
    
    public class JambalayaClsesTreeDragSourceListener extends ClsesTreeDragSourceListener {
    	
        /** 
         * Overridden so that we can drag and drop :THING
         * and so that we can't drag and drop things that won't show in jambalaya.
         */
        public boolean canStartDrag(Collection objects) {
            boolean canStartDrag = true;
            ShrimpProject project = JambalayaProjectPlugin.getJambalayaProjectPlugin().getJambalayaProject();
            if (project != null) {
                try {
                    ProtegeDataBean protegeDataBean = (ProtegeDataBean) project.getBean(ShrimpProject.DATA_BEAN);
                    Iterator i = objects.iterator();
                    while (i.hasNext()) {
                        Frame frame = (Frame) i.next();
                        if (!protegeDataBean.shouldImportFrameAsArtifact(frame, false, false, true)) {
                            canStartDrag = false;
                            break;
                        }
                    }
                } catch (BeanNotFoundException e) {
                    e.printStackTrace();
                }
            }
            return canStartDrag;
        }
    }    
}