/**
 * Copyright 1998-2007, CHISEL Group, University of Victoria, Victoria, BC, Canada.
 * All rights reserved.
 */
package ca.uvic.csr.shrimp.jambalaya;

import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import javax.swing.JButton;
import javax.swing.JComponent;

import ca.uvic.csr.shrimp.ShrimpApplication.BeanNotFoundException;
import ca.uvic.csr.shrimp.ShrimpProject.ShrimpProject;
import ca.uvic.csr.shrimp.gui.ShrimpTransferHandler;
import ca.uvic.csr.shrimp.jambalaya.ProtegeDataBean.ProtegeDataBean;
import edu.stanford.smi.protege.model.Cls;
import edu.stanford.smi.protege.model.Frame;
import edu.stanford.smi.protege.util.TransferableCollection;

/**
 * Handles drag and drop transfers from Protege to Jambalaya
 * 
 * @author Rob Lintern
 */
public class JambalayaTransferHandler extends ShrimpTransferHandler {

	private ShrimpProject project;
	
	public JambalayaTransferHandler(ShrimpProject project) {
		this.project = project;
	}
	
    /* (non-Javadoc)
     * @see javax.swing.TransferHandler#canImport(javax.swing.JComponent, java.awt.datatransfer.DataFlavor[])
     */
    public boolean canImport(JComponent comp, DataFlavor[] transferFlavors) {
        boolean canImport = false;
        for (int i = 0; i < transferFlavors.length && !canImport; i++) {
            DataFlavor flavor = transferFlavors[i];
            if (flavor.equals(TransferableCollection.getCollectionFlavor())) {
                canImport = true;
            }
        }
        return canImport; 
    }
    
    /* (non-Javadoc)
     * @see javax.swing.TransferHandler#importData(javax.swing.JComponent, java.awt.datatransfer.Transferable)
     */
    public boolean importData(JComponent comp, Transferable t) {
        if (t.isDataFlavorSupported(TransferableCollection.getCollectionFlavor())) {
            try {
                Collection sources = (Collection) t.getTransferData(TransferableCollection.getCollectionFlavor());
                Set rootClses = new HashSet();
                for (Iterator iter = sources.iterator(); iter.hasNext();) {
                    Frame frame = (Frame) iter.next();
                    if (frame instanceof Cls) {
                        rootClses.add(frame);
                    }
                }
                if (!rootClses.isEmpty()) {
                    try {
                        if (project != null) {
	                        ProtegeDataBean dataBean = (ProtegeDataBean) project.getBean (ShrimpProject.DATA_BEAN);   
	                        dataBean.setRootClses(rootClses);
						
	                        if (comp instanceof JButton) {
	                            ((JButton)comp).doClick(); // this should hopefully refresh the display properly
	                        }
                        }
                    } catch (BeanNotFoundException e) {
                        e.printStackTrace();
                    }
                }
            } catch (Exception ex) {
                ex.printStackTrace();
            }
        } 
        return false; //always return false so that no data gets "moved" from Protege
    }
    
}
